/*

  File Inverse.ccp:

  find the inverse of a 3 x 3 given matrix by using the
  LU factorization.

*/


#include <iostream>
using namespace std;

#include "LUvect.h"
#include "LUmat.h"


int main()
{
  cout << "Computes the inverse of a matrix through the LU factorization" << endl;

  const int N = 3;
  

  // Instantiates the object myMat of class LUmat and defines it
  // then copy it in a new matrix for the LU factorization

  LUmat omyMat = LUmat( N );
  LUmat myMat = LUmat( N );
  omyMat.setElement( 1, 1, 3.0 );
  omyMat.setElement( 1, 2, 2.0 );
  omyMat.setElement( 1, 3, 1.0 );
  omyMat.setElement( 2, 1, 4.0 );
  omyMat.setElement( 2, 2, 5.0 );
  omyMat.setElement( 2, 3, 2.0 );
  omyMat.setElement( 3, 1, 1.0 );
  omyMat.setElement( 3, 2, 2.0 );
  omyMat.setElement( 3, 3, 3.0 );
  myMat.duplicateMat( omyMat );


  // Instantiates the object invMat (inverse of omyMat) and uniMat (Identity matrix) of class LUmat

  LUmat invMat = LUmat( N );
  LUmat uniMat = LUmat( N );


  // Prints the matrix of coefficients on the screen

  cout << "Initial matrix:" << endl;
  for ( int i = 1; i <= N; i++ )
    {
      for ( int j = 1; j <= N; j++ ) cout << "   " << omyMat.getElement( i, j );
      cout << endl;
    }


  // Lu factorization of the matrix

  myMat.luFact( );

  
  // Defines the column vectors for the inverse (x) and identity matrices (b)

  LUvect x = LUvect( N );
  LUvect b = LUvect( N );

  // For each column of the inverse defines the RHS and finds the solution
  
  for ( int j = 1; j <= N; j++ )
    {
      // Defines the i-th column of identity matrix
      for ( int i = 1; i <= N; i++ ) b.setElement( i, 0.0 );
      b.setElement( j, 1.0 );
      
      // Solves the system
      myMat.luSolve( x, b );

      // Copies the result in the inverse matrix
      for ( int i = 1; i <= N; i++ ) invMat.setElement( i, j, x.getElement( i ) );
    }


  // Prints on the screen the inverse of the matrix

  cout << "Inverse of the matrix:" << endl;
  for ( int i = 1; i <= N; i++ )
    {
      for ( int j = 1; j <= N; j++ ) cout << "   " << invMat.getElement( i, j );
      cout << endl;
    }


  // Verify the result. The product myMat*invMat is stored in uniMat

  omyMat.matMatProduct( invMat, uniMat );
  cout << "Product matrix by inverse:" << endl;
  for ( int i = 1; i <= N; i++ )
    {
      for ( int j = 1; j <= N; j++ ) cout << "   " << uniMat.getElement( i, j );
      cout << endl;
    }


  // Verify the result. The product invMat*myMat is stored in uniMat

  invMat.matMatProduct( omyMat, uniMat );
  cout << "Product inverse by matrix:" << endl;
  for ( int i = 1; i <= N; i++ )
    {
      for ( int j = 1; j <= N; j++ ) cout << "   " << uniMat.getElement( i, j );
      cout << endl;
    }

  return 0;
}
