/*

  File LUmat.h:

  defines the class LUmat, to solve a nxn linear system
  through the LU factorization algorithm.

*/


#include "LUvect.h"

#ifndef LUmat_H

#define LUmat_H

class LUmat
{

 private:

  int dim;                                                 // Dimension of the matrix nxn
  double ** omat;                                          // Original matrix
  double ** mat;                                           // Factorized matrix
  double * tmp;                                            // temporary vector to store intermediate results

 public:

  LUmat( int );                                            // Class constructor
  void luFact( );                                          // LU factorization
  void luSolve( LUvect, LUvect );                          // Solves the system with the factorized matrix
                                                           // given the known terms
  void setElement( int, int, double );                     // Initialize a single element of the matrix 
  double getElement( int , int );                          // Returns the ij-th element of the matrix
  void matVectProduct( LUvect, LUvect );                   // Compute the product of the current matrix by
                                                           // the first vector and returns the result in the
                                                           // second vector
  void matMatProduct( LUmat, LUmat );                      // Compute the product of the current matrix by
                                                           // the first argument matrix and returns the result 
                                                           // in the second argument matrix
  void duplicateMat( LUmat );                              // Copy another matrix in the current matrix
  
};

#endif
