/*

  File Sys_ndd.ccp:

  This program is identical to Sys_scal.cpp, except for the fact that
  the matrix is NOT any more diagonally dominant:

  a_ij = 1.0/(1.0+i+j)

  (I removed the condition a_ii = 1!).

  Then, we change N and verify that, by increasing N, the results get worse.

  Try, for instance, N = 3, 8, 20 and see how the results becomes less and less
  precise.

*/


#include <iostream>
using namespace std;

#include "LUvect.h"
#include "LUmat.h"


int main()
{
  cout << "Solve a system of linear equations through the LU factorization" << endl;

  int N;
  cout << "Insert the value of N: ";
  cin >> N;


  // Instantiates the object omat and mat of class LUmat and define them

  LUmat omyMat = LUmat( N );
  LUmat myMat = LUmat( N );
  for ( int i = 1; i <= N; i++ )
    for ( int j = 1; j <= N; j++ )
      {
	omyMat.setElement( i, j, 1.0 / ( 1.0 + i + j ) );
      }
  myMat.duplicateMat( omyMat );
  

  // Prints the matrix of coefficients on the screen

  cout.precision( 15 );     // Sets output precision with 15 decimal figures
  cout << "Initial matrix:" << endl;
  for ( int i = 1; i <= N; i++ )
    {
      for ( int j = 1; j <= N; j++ ) cout << "   " << omyMat.getElement( i, j );
      cout << endl;
    }


  // Lu factorization of the matrix

  myMat.luFact( );

  
  // Defines the vectors for the solution (x) and known terms (b)

  LUvect x = LUvect( N );
  LUvect b = LUvect( N );

  for ( int i = 1; i <= N; i++ )
    {
      b.setElement( i, ( i -1 ) * 1.0 );
    }


  // Prints on the screen the RHS of the system

  cout << "RHS vector:" << endl;
  for ( int i = 1; i <= N; i++ ) cout << "   " << b.getElement( i );
  cout << endl;


  // Solves the system

  myMat.luSolve( x, b );

  
  // Prints on the screen the solution of the system

  cout << "Vector of the solution:" << endl;
  for ( int i = 1; i <= N; i++ ) cout << "   " << x.getElement( i );
  cout << endl;


  // Verify the result. The product A*x is stored in b

  cout << "Product of the matrix by the solution vector:" << endl;
  omyMat.matVectProduct( x, b );
  for ( int i = 1; i <= N; i++ ) cout << "   " << b.getElement( i );
  cout << endl;

  return 0;
}
