/*

  File System.ccp:

  solves a 3 x 3 linear system of equations given the matrix of the
  coefficients and the RHS vector of known terms by using the
  LU factorization.

*/


#include <iostream>
using namespace std;

#include "LUvect.h"
#include "LUmat.h"


int main()
{
  cout << "Solve a system of linear equations through the LU factorization" << endl;

  const int N = 3;
  

  // Instantiates the object mat of class LUmat and defines it
  // then copy it in a new matrix for the LU factorization

  LUmat omyMat = LUmat( N );
  LUmat myMat = LUmat( N );
  omyMat.setElement( 1, 1, 3.0 );
  omyMat.setElement( 1, 2, 2.0 );
  omyMat.setElement( 1, 3, 1.0 );
  omyMat.setElement( 2, 1, 4.0 );
  omyMat.setElement( 2, 2, 5.0 );
  omyMat.setElement( 2, 3, 2.0 );
  omyMat.setElement( 3, 1, 1.0 );
  omyMat.setElement( 3, 2, 2.0 );
  omyMat.setElement( 3, 3, 3.0 );
  myMat.duplicateMat( omyMat );
  

  // Prints the matrix of coefficients on the screen

  cout << "Initial matrix:" << endl;
  for ( int i = 1; i <= N; i++ )
    {
      for ( int j = 1; j <= N; j++ ) cout << "   " << omyMat.getElement( i, j );
      cout << endl;
    }


  // Lu factorization of the matrix

  myMat.luFact( );

  
  // Defines the vectors for the solution (x) and known terms (b)

  LUvect x = LUvect( N );
  LUvect b = LUvect( N );

  for ( int i = 1; i <= N; i++ )
    {
      b.setElement( i, ( i -1 ) * 1.0 );
    }


  // Prints on the screen the RHS of the system

  cout << "RHS vector:" << endl;
  for ( int i = 1; i <= N; i++ ) cout << "   " << b.getElement( i );
  cout << endl;


  // Solves the system

  myMat.luSolve( x, b );

  
  // Prints on the screen the solution of the system

  cout << "Vector of the solution:" << endl;
  for ( int i = 1; i <= N; i++ ) cout << "   " << x.getElement( i );
  cout << endl;


  // Verify the result. The product A*x is stored in b

  cout << "Product of the matrix by the solution vector:" << endl;
  omyMat.matVectProduct( x, b );
  for ( int i = 1; i <= N; i++ ) cout << "   " << b.getElement( i );
  cout << endl;

  return 0;
}
