/*

  File Tridiag.ccp:

  solves a 3x3 tridiagonal system of linear equations
  by using the Thomas' algorithm for the LU factorization

*/


#include <iostream>
#include "LUtrid.h"
#include "LUvect.h"
using namespace std;


main()
{
  cout << "Computes the solution of a 3x3 tridiagonal system of equations" << endl;


  // Instantiate an object omyMat and myMat of class LUtrid

  const int N = 3;
  LUtrid omyMat = LUtrid( N );
  LUtrid myMat = LUtrid( N );
  omyMat.setElement( 1, 1, 4.0 );
  omyMat.setElement( 1, 2, 1.0 );
  omyMat.setElement( 1, 3, 0.0 );
  omyMat.setElement( 2, 1, 2.0 );
  omyMat.setElement( 2, 2, 5.0 );
  omyMat.setElement( 2, 3, 2.0 );
  omyMat.setElement( 3, 1, 0.0 );
  omyMat.setElement( 3, 2, 1.0 );
  omyMat.setElement( 3, 3, 2.0 );
  myMat.duplicateMat( omyMat );


  // Prints the initial matrix on the screen

  cout.precision( 15 );
  cout << "Initial matrix:" << endl;
  for ( int i = 1; i <= N; i++ )
    {
      for ( int j = 1; j <= N; j++ ) cout << "   " << omyMat.getElement( i, j );
      cout << endl;
    }

  // LU factorization of the matrix

  myMat.luFact( );


  // Defines the vectors for the solution and the RHS of the system

  LUvect x = LUvect( N );
  LUvect b = LUvect( N );

  for ( int i = 1; i <= N; i++ )
    {
      b.setElement( i, ( i -1 ) * 1.0 );
    }


  // Prints the RHS vector on the screen

  cout << "Right hand side vector:" << endl;
  for ( int i = 1; i <= N; i++ ) cout << "   " << b.getElement( i );
  cout << endl;


  // Solves the LU system

  myMat.luSolve( x, b );


  // Prints the result

  cout << "Vector of the solution:" << endl;
  for ( int i = 1; i <= N; i++ ) cout << "   " << x.getElement( i );
  cout << endl;


  // Verifies the result

  omyMat.matVectProduct( x, b );
  cout << "Row by column product of the matrix times the solution:" << endl;
  for ( int i = 1; i <= N; i++ ) cout << "   " << b.getElement( i );
  cout << endl;

  return 0;

}
